//========================================================================
//=
//=     *** Kemper ASIC ***					2/11/2010
//=
//=
//=     Source code from Kemper LED adapted to Jimmy Neutron Pinewood car.
//=		All the comm source was removed. The LEDs were setup to simply come
//=		on and blink. 
//========================================================================

#include "Jimmy.h"

// ----------------------------------------------------------------------------


// Keep a threshold for each LED. When the timer rolls by the threshold ServiceLEDs
// will change the output pin. 
byte iBlue, iGreen, iRed, iWhite;

// These are the target values for each color.
byte iBlueTar, iGreenTar, iRedTar, iWhiteTar;

// Rate at which the LEDs are ramped up and down.
byte iRBlue,    iDBlue;
byte iRGreen,   iDGreen;
byte iRRed,     iDRed;
byte iRWhite,   iDWhite;


// Keep the declaration of booleans all together so the compiler can pack them
// into one byte.
bool fStretch, fTmp;

byte iDwell, iDwellTmr, iPulLED;

// Add some Pinewood Hacks
const byte iRedMax = 0xFF;		// Max intensity for the red LED.
const byte iRedMin = 0x80;		// Min      "     "   "   "   "
const byte iRRedUp = 0x05;		// Ramp up rate.
const byte iRRedDwn = 0x03;		// Ramp down rate.

const byte iWhtMax = 0xFF; 
const byte iWhtMin = 0x00; 
const byte iRwhtUp = 0x05; 
const byte iRWhtDwn = 0xFF;

const byte iBluMax = 0xFF; 
const byte iBluMin = 0x00; 
const byte iRBluUp = 0x10; 
const byte iRBluDwn = 0x10;

const byte iGrnMax = 0xFF; 
const byte iGrnMin = 0x00; 
const byte iRGrnUp = 0x10; 
const byte iRGrnDwn = 0x10;

//========================================================================
// Main Loop...
//========================================================================
void main() {
	byte cmd, i;

	init();				  	// Initialize everything.


	// Setup default flash rates for Pinewood Car.
	// Init all the threshholds and targets.

	iRedTar = iRedMax;		// Set target intensity to get the ball rolling.
	iRRed = iRRedUp;		// Use a defined ramp rate.
	iDRed = iRRedDwn;

	iWhiteTar = iWhtMax;
	iRWhite = iRWhtUp;
	iDWhite = iRWhtDwn;

	iBlueTar = iBluMax;
	iRBlue = iRBluUp;
	iDBlue = iRBluDwn;
	iBlue = iBluMin;

	iGreenTar = iGrnMax;
	iRGreen = iRGrnUp;
	iDGreen = iRGrnDwn;
	iGreen = iGrnMax;

	while ( 1 ) {

		vServiceLEDs();		// No mater what, keep the LEDs updating!

		// On timer0 overflow (approx. every 16.5ms) ramp LED(s)...
		// This is the lowest priority task in the system. Nobody will
		// notice a ramp that takes just a bit longer.
		if ( T0IF ) {
			T0IF = false;	// Clear timer0 overflow flag.

			vServiceLEDs();
			vRampLED( &iRed, &iRedTar, iRRed, iDRed, 0x08 );

			vServiceLEDs();
			vRampLED( &iGreen, &iGreenTar, iRGreen, iDGreen, 0x04 );

			vServiceLEDs();
			vRampLED( &iBlue, &iBlueTar, iRBlue, iDBlue, 0x02 );

			vServiceLEDs();
			vRampLED( &iWhite, &iWhiteTar, iRWhite, iDWhite, 0x01 );

			fStretch = false;
		}

		// Jump the timer count back - this has the effect of stretching the
		// timer from 256 counts to 260. This makes the roll over every 60Hz 
		// instead of 61Hz. This becomes important for video cameras. The human
		// eye won't see an adjustment around 0xA0 counts. The eye will see the
		// adjustment if made around zero!
		if ( (TMR0 & 0xA0) && (fStretch == false) ) {
			TMR0 -= 4;
			fStretch = true;			// Flag that stretch is complete.
		}


		// Pinewood Hack Starts Here ............

		// Manage cycling the LEDs between min and max values. This is a hack
		// but it should work OK.
        if ( iRed == iRedMax)		iRedTar = iRedMin;
        if ( iRed == iRedMin)		iRedTar = iRedMax;

		if ( iWhite == iWhtMax)		iWhiteTar = iWhtMin;
		if ( iWhite == iWhtMin)		iWhiteTar = iWhtMax;

		if ( iBlue == iBluMax)		iBlueTar = iBluMin;
		if ( iBlue == iBluMin)		iBlueTar = iBluMax;

		if ( iGreen == iGrnMax)		iGreenTar = iGrnMin;
		if ( iGreen == iGrnMin)		iGreenTar = iGrnMax;
	}     
}


// Given some pointers and rate rates, this function makes one step
// change adjustment to the output threshold. This function should be 
// called every 16.384ms. Example, if ramp equaled one, and thresh started
// at zero, it would take over 4 seconds to ramp the threshold from zero 
// to 0xFF at 16.384 ms per step.
//========================================================================
void vRampLED( byte *thresh, byte *tar, ramp, decay, byte iIndex ) {

	// If threshold equals target then do nothing.

	// If threshold is below target then use "ramp" to bring up the threshold.
	if ( *thresh < *tar ) {
		*thresh += ramp;                               
	// Use the status bit overflow to check if we rolled over.
		if ( MATHOF || ( *thresh >= *tar) )	*thresh = *tar;
	}

	// If threshold is above the target then use "decay" to bring down the threshold.
	if ( *thresh > *tar ) {
		*thresh -= decay;                               
	// Use the status bit overflow to check if we went negative.
		if ( ~MATHOF || ( *thresh <= *tar) ) *thresh = *tar;
	}

	// If pulsing an led is true...
	if ( iPulLED & iIndex ) {
		// Once the led is at the target level start counting down the timer.
		if ( (*thresh == *tar) && iDwellTmr ) --iDwellTmr;
		// Once the timer times out, jump the target to zero and turn off the pulse bit.
		if ( iDwellTmr == 0 ) {
			*tar = 0;
			iPulLED &= ~iIndex;
		}
	}
}


// Compare the LED threshold to the current value in Timer0. On a threshold
// greater than Timer0, then set the output pin high. Else, set the pin 
// low. Note, if the threshold is zero then the output is always low 
// regardless of the timer. Also, in the same vain, a threshold of 0xFF 
// will allways have the output pin high. This makes me happy when I look
// at my logic analyzer.
// Note, timer0 is couting at 64uS per tick. Therefore, this function must 
// be called more often than that if the LEDs are to look nice.
//
// Call this fuction often! 
// Also, call this fuction from inside interrupts (it's OK and safe)!
//========================================================================
void vServiceLEDs( void ) {

	if ( iBlue == 0 )			pBLU = true;
	else if ( iBlue >= TMR0 )	pBLU = false;
	else						pBLU = true;

	if ( iGreen == 0 )			pGRN = true;
	else if ( iGreen >= TMR0 )	pGRN = false;
	else						pGRN = true;

	if ( iRed == 0 )			pRED = true;
	else if ( iRed >= TMR0 )	pRED = false;
	else						pRED = true;

	if ( iWhite == 0 )			pWHT = true;
	else if ( iWhite >= TMR0 )	pWHT = false;
	else						pWHT = true;
}



// Initialize everything! Called after powerup.
//========================================================================
void init( void ) {
	byte i;

	disable_interrupts( GLOBAL );	// Disable global interrupts.

	ANSEL = 0x00;					// All digital - No Analog Pins.
	GPIO = 0b110111;				// Set all IO pins at once - LEDs are active low.
	SET_TRIS_A( 0b001100);			// 0=Output -> Bit Order: A5 A4 - A3 A2 A1 A0

	// Enable gloabl weak pullups (active low). Only pin this affects 
	// is GP2 (the comm pin). All other pins are outputs. GP3/MCLR is ignored.
	GPPU = 0;                                     
	WPU = PULLUP;					// Default weak pull-up on pCOMM.
	IOC = 0;						// Interrupt on change can't be trusted!

	// 500nS / Instruction
	// 1 / ((8Mhz Clock / 4 Cycles/Instruction) / 128 Prescaler) = 64uS / Tick
	setup_timer_0( RTCC_INTERNAL | RTCC_DIV_128 );		// Rollover = 16.384ms

	// 1 / ((8Mhz Clock / 4 Cycles/Instruction) / 8 Prescaler) = 4uS / Tick
	setup_timer_1( T1_INTERNAL | T1_DIV_BY_8 );

	// Init all the threshholds and targets.
	iRed = iGreen = iBlue = iWhite = 0;
	iRedTar = iGreenTar = iBlueTar = iWhiteTar = 0;

	// Init ramp rates to a nice rate.
	iRRed = iRGreen = iRBlue = iRWhite = 0x15;			// Ramp up rates.
	iDRed = iDGreen = iDBlue = iDWhite = 0x15;			// Decay down rates.

	iDwell = 0x10; iDwellTmr = 0; iPulLED = 0;

	set_timer1( 0 );				// Clear timeout timer.

	GPIF = 0;						// Clear the I/O pin changed irq flag.
	INTF = 0;						// Clear the INT request flag.
	PIE1 = 0x00;					// No other interrupts allowed!
}



