//=============================================================================
//=
//=
//=
//=
//=============================================================================

#include <12F609.h>

#pragma fuses IOSC8, MCLR, INTRC_IO, PROTECT

#use delay(clock=8M)
#use fast_io( ALL )


// ========================================================
// Note, all types (except floats) are unsiged by default!
// ========================================================
//typedef unsigned int8 uint8;
//typedef unsigned int16 uint16;
typedef unsigned int1 bool;

#BYTE	TMR0 	= 0x01		// Timer0 Register
#BYTE	STATUS 	= 0x03		// Status Register
#BIT 	MATHOF 	= 0x03.0	// Math Over/Under flow bit in status register.
#BYTE 	GPIO 	= 0x05		// General I/O Port
#BIT 	GP0  	= 0x05.0	// LED 0
#BIT 	GP1  	= 0x05.1	// LED 1
#BIT 	GP2  	= 0x05.2	// Comm
#BIT 	GP3		= 0x05.3	// MCLR / Unused
#BIT 	GP4  	= 0x05.4	// LED 2
#BIT 	GP5  	= 0x05.5	// LED 3
#BYTE	PCLATH 	= 0x0A		// PC Latch
#BYTE	INTCON 	= 0x0B		// INTERRUPT CONTROL REGISTER
#BIT	GPIF 	= 0x0B.0	// GPIO Change Interrupt Flag bit
#BIT	INTF	= 0x0B.1	// GP2/INT External Interrupt Flag bit
#BIT 	T0IF 	= 0x0B.2	// Timer0 Overflow Flag
#BIT	GIE 	= 0x0B.7	// Global Interrupt Enable bit
#BYTE 	PIR1 	= 0x0C		// PERIPHERAL INTERRUPT REQUEST REGISTER 1
#BYTE	TMR1L 	= 0x0E		// Timer1 Low Byte
#BYTE	TMR1H 	= 0x0F		// Timer1 High Byte
#BIT	GPPU	= 0x81.7	// Global Pullup Enable (active low)
#BYTE	TRISIO	= 0x85		// GPIO TRI-STATE REGISTER ( 1's=Input)
#BYTE	PIE1 	= 0x8C		// PERIPHERAL INTERRUPT ENABLE REGISTER 1
#BYTE	WPU		= 0x95		// Weak Pullup Control register.
#BYTE	IOC 	= 0x96		// Interrupt-on-Change GPIO Register
#BYTE	ANSEL 	= 0x9F		// Analog Select Register

// More human readable / memorable names.
#BIT 	pBLU  	= 0x05.0	// LED 0 - Blue
#BIT 	pGRN  	= 0x05.1	// LED 1 - Green
#BIT 	pCOMM  	= 0x05.2	// Comm Pin
#BIT 	pRED  	= 0x05.4	// LED 2 - Red
#BIT 	pWHT  	= 0x05.5	// LED 3 - White

// Data Direction Control Bit for Comm Pin.
#BIT	pDDR_COMM	= 0x85.2
#define IN			1
#define OUT			0

#define PULLUP	0x04		// Bit position of COMM pin (see WPU).

// Bit mask for Rx commands.
#define mRED	0x80
#define mGRN	0x40
#define mBLU	0x20
#define mWHT	0x10

#define NACK	0x00		// Negative response.

// Timer1 rates (4us per tick).
#define m2ms	500			// Would be nice to be able to define as (2ms / 4us)
#define m60ms	15000		// but stupid-o compiler chokes on that!

// Define all the commands.
enum eCommands { 
	RESET				= 0,
	SET_PULLUP			= 1,
	SET_LED_ON_OFF		= 2,
	QUICK_SET_LED_LEVEL	= 3,
	SET_LED_RAMP_DECAY	= 4,
	SET_LED_LEVEL		= 5,
	START_TURBO_PING	= 6,
	RESET_PWM_TIMER		= 7,
	SET_ACK_RESPONSE	= 8,
	SET_2ND_ADDR		= 9,
	PULSE_LED			= 10
};


#define LEVELS	16
const byte bLevels[ LEVELS ] = {
	0,		4,		13,		20,
	28,		35,		48,		59,
	72,		88,		104,	120,
	143,	167,	199,	255
};

// Function protos...
void init( void );				// Call to initialize everything.    
void vGetCharISR( void );		// Irq for new characters.	     
void vServiceLEDs( void );		// Keeps LEDs updated.		     
bool fChecksum( byte len );		// Compute checksum.
void vParsePacket( void );		// Parse the new packet.	     
void Tx( byte ch );				// Transmit a character onto the bus.

// Ramp the indexed LED.
void vRampLED(  byte *thersh, byte *tar, byte ramp, byte decay, byte iIndex );	




